<!-- rmarkdown v1 -->

<!-- to compile this: library("knitr"); knit2html("rnaseqGene.Rmd") -->

# RNA-Seq workflow: gene-level exploratory analysis and differential expression 

Michael Love [1], Simon Anders [2], Wolfgang Huber [2]

[1] Department of Biostatistics, Dana-Farber Cancer Institute and
Harvard School of Public Health, Boston, US; 

[2] European Molecular Biology Laboratory (EMBL), Heidelberg, Germany.

<script type="text/javascript">
document.addEventListener("DOMContentLoaded", function() {
  document.querySelector("h1").style.marginTop = "0";
});
</script>
<script type="text/javascript">
document.addEventListener("DOMContentLoaded", function() {
  var links = document.links;  
  for (var i = 0, linksLength = links.length; i < linksLength; i++)
    if (links[i].hostname != window.location.hostname)
      links[i].target = '_blank';
});
</script>
<style type="text/css" scoped>
body, td {
   font-family: sans-serif;
   background-color: white;
   font-size: 13px;
}

body {
  max-width: 800px;
  margin: 0 auto;
  padding: 1em 1em 2em;
  line-height: 20px;
}

/* element spacing */

p, pre { 
  margin: 0em 0em 1em;
}

/* center images and tables */
img, table {
  margin: 0em auto 1em;
}

p {
  text-align: justify;
}

tt, code, pre {
   font-family: 'DejaVu Sans Mono', 'Droid Sans Mono', 'Lucida Console', Consolas, Monaco, monospace;
}

h1, h2, h3, h4, h5, h6 { 
  font-family: Helvetica, Arial, sans-serif;
  margin: 1.2em 0em 0.6em 0em;
  font-weight: bold;
}

h1 {
  font-size: 250%;
  font-weight: normal;
  color: #87b13f;
  line-height: 1.1em;
}

h2 {
  font-size: 160%;
  font-weight: normal;
  line-height: 1.4em;
  border-bottom: 1px #1a81c2 solid;
}

h3 {
  font-size: 130%;  
}

h2, h3 {
  color: #1a81c2;
}

h4, h5, h6 {
  font-size:115%;
} /* not expecting to dive deeper than four levels on a single page */

/* links are simply blue, hovering slightly less blue */
a { color: #1a81c2; }
a:active { outline: none; }
a:visited { color: #1a81c2; }
a:hover { color: #4c94c2; }

pre, img {
  max-width: 100%;
  display: block;
}

pre {
  border: 0px none;
  background-color: #F8F8F8;
  white-space: pre;
  overflow-x: auto;
}

pre code {
  border: 1px #aaa dashed;
  background-color: white;
  display: block;
  padding: 1em;  
  color: #111;
  overflow-x: inherit;
}

/* markdown v1 */
pre code[class] {
  background-color: inherit;
}

/* markdown v2 */
pre[class] code {
  background-color: inherit;
}



/* formatting of inline code */
code { 
  color: #87b13f;
  font-size: 92%;
}

/* formatting of tables */

table, td, th {
  border: none;
  padding: 0 0.5em;
}

/* alternating row colors */
tbody tr:nth-child(odd) td {
  background-color: #F8F8F8;
}

blockquote {
   color:#666666;
   margin:0;
   padding-left: 1em;
   border-left: 0.5em #EEE solid;
}

hr {
   height: 0px;
   border-bottom: none;
   border-top-width: thin;
   border-top-style: dotted;
   border-top-color: #999999;
}

@media print {
   * {
      background: transparent !important;
      color: black !important;
      filter:none !important;
      -ms-filter: none !important;
   }

   body {
      font-size:12pt;
      max-width:100%;
   }

   a, a:visited {
      text-decoration: underline;
   }

   hr {
      visibility: hidden;
      page-break-before: always;
   }

   pre, blockquote {
      padding-right: 1em;
      page-break-inside: avoid;
   }

   tr, img {
      page-break-inside: avoid;
   }

   img {
      max-width: 100% !important;
   }

   @page :left {
      margin: 15mm 20mm 15mm 10mm;
   }

   @page :right {
      margin: 15mm 10mm 15mm 20mm;
   }

   p, h2, h3 {
      orphans: 3; widows: 3;
   }

   h2, h3 {
      page-break-after: avoid;
   }
}
</style>

This lab will walk you through an end-to-end RNA-Seq differential
expression workflow, using *[DESeq2](http://bioconductor.org/packages/release/bioc/html/DESeq2.html)* along with other
Bioconductor packages.  We will start from the FASTQ files, show how
these were aligned to the reference genome, prepare gene expression
values as a count matrix by counting the sequenced fragments, perform
exploratory data analysis (EDA), perform differential gene expression
analysis with *DESeq2*, and visually explore the results.

We note that a number of other Bioconductor packages can also be used for
statistical inference of differential expression at the gene level including
*[edgeR](http://bioconductor.org/packages/release/bioc/html/edgeR.html)*, *[BaySeq](http://bioconductor.org/packages/release/bioc/html/BaySeq.html)*, *[DSS](http://bioconductor.org/packages/release/bioc/html/DSS.html)* and
*[limma](http://bioconductor.org/packages/release/bioc/html/limma.html)*.

## Experimental data

The data used in this workflow is an RNA-Seq experiment of airway
smooth muscle cells treated with dexamethasone, a synthetic
glucocorticoid steroid with anti-inflammatory effects. Glucocorticoids
are used, for example, in asthma patients to prevent or reduce
inflammation of the airways. In the experiment, four primary human
airway smooth muscle cell lines were treated with 1 micromolar
dexamethasone for 18 hours. For each of the four cell lines, we have a
treated and an untreated sample. The reference for the experiment is:

Himes BE, Jiang X, Wagner P, Hu R, Wang Q, Klanderman B, Whitaker RM,
Duan Q, Lasky-Su J, Nikolos C, Jester W, Johnson M, Panettieri R Jr,
Tantisira KG, Weiss ST, Lu Q. "RNA-Seq Transcriptome Profiling
Identifies CRISPLD2 as a Glucocorticoid Responsive Gene that Modulates
Cytokine Function in Airway Smooth Muscle Cells." PLoS One. 2014 Jun
13;9(6):e99625.
PMID: [24926665](http://www.ncbi.nlm.nih.gov/pubmed/24926665).
GEO: [GSE52778](http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE52778).

## Preparing count matrices

As input, the *DESeq2* package expects count data as obtained,
e.g., from RNA-Seq or another high-throughput sequencing experiment,
in the form of a matrix of integer values. The value in the *i*-th row
and the *j*-th column of the matrix tells how many reads have been
mapped to gene *i* in sample *j*.  Analogously, for other types of
assays, the rows of the matrix might correspond e.g., to binding
regions (with ChIP-Seq) or peptide sequences (with quantitative mass
spectrometry).

The count values must be raw counts of sequencing reads. This is
important for *DESeq2*'s statistical model to hold, as only the
actual counts allow assessing the measurement precision
correctly. Hence, please do not supply other quantities, such as
(rounded) normalized counts, or counts of covered base pairs -- this
will only lead to nonsensical results.

## Aligning reads to a reference

The computational analysis of an RNA-Seq experiment begins earlier:
what we get from the sequencing machine is a set of FASTQ files that
contain the nucleotide sequence of each read and a quality score at
each position.  These reads must first be aligned to a reference
genome or transcriptome.  It is important to know if the sequencing
experiment was single-end or paired-end, as the alignment software
will require the user to specify both FASTQ files for a paired-end
experiment. The output of this alignment step is commonly stored in a
file format called [SAM/BAM](http://samtools.github.io/hts-specs).

A number of software programs exist to align reads to the reference
genome, and the development is too rapid for this document to provide
an up-to-date list. We recommend consulting benchmarking papers that
discuss the advantages and disadvantages of each software, which
include accuracy, ability to align reads over splice junctions, speed,
memory footprint, and many other features.

The reads for this experiment were aligned to the Ensembl release 75 human reference
genome using the
[STAR read aligner](https://code.google.com/p/rna-star/):

```
for f in `cat files`; do STAR --genomeDir ../STAR/ENSEMBL.homo_sapiens.release-75 \
--readFilesIn fastq/$f\_1.fastq fastq/$f\_2.fastq \
--runThreadN 12 --outFileNamePrefix aligned/$f.; done
```

[SAMtools](http://samtools.sourceforge.net/) was used to generate BAM files.

```
cat files | parallel -j 7 samtools view -bS aligned/{}.Aligned.out.sam -o aligned/{}.bam
```

The BAM files for a number of sequencing runs can then be used to
generate count matrices, as described in the following section.

## Example BAM files

Besides the main count matrix, which we will use later, the
*[airway](http://bioconductor.org/packages/release/data/experiment/html/airway.html)* package also contains a small subset of the raw
data, namely eight BAM file each with a subset of the reads. We will
use these files to demonstrate how a count matrix can be constructed
from BAM files. Afterwards, we will load the full count matrix
corresponding to all samples and all data, which is already provided
in the same package, and will continue the analysis with that full
table.

We load the data package with the example data:


    library("airway")

The R function *system.file* can be used to find out where on your
computer the files from a package have been installed. Here we ask for
the full path to the `extdata` directory, which is part of the
*[airway](http://bioconductor.org/packages/release/data/experiment/html/airway.html)* package.


    dir <- system.file("extdata", package="airway", mustWork=TRUE)

In this directory, we find the eight BAM files (and some other files):


    list.files(dir)

    ##  [1] "GSE52778_series_matrix.txt"        "Homo_sapiens.GRCh37.75_subset.gtf"
    ##  [3] "sample_table.csv"                  "SraRunInfo_SRP033351.csv"         
    ##  [5] "SRR1039508_subset.bam"             "SRR1039509_subset.bam"            
    ##  [7] "SRR1039512_subset.bam"             "SRR1039513_subset.bam"            
    ##  [9] "SRR1039516_subset.bam"             "SRR1039517_subset.bam"            
    ## [11] "SRR1039520_subset.bam"             "SRR1039521_subset.bam"

Typically, we have a table with experimental metadata for our
samples. For your own project, you might create such a comma-separated
value (CSV) file using a text editor or spreadsheet software such as Excel.

We load this file with *read.csv*. The parentheses around the last
line are used to print the result in addition to storing it to the
`sampleTable` object.


    csvfile <- file.path(dir,"sample_table.csv")
    (sampleTable <- read.csv(csvfile,row.names=1))

    ##            SampleName    cell   dex albut        Run avgLength Experiment    Sample    BioSample
    ## SRR1039508 GSM1275862  N61311 untrt untrt SRR1039508       126  SRX384345 SRS508568 SAMN02422669
    ## SRR1039509 GSM1275863  N61311   trt untrt SRR1039509       126  SRX384346 SRS508567 SAMN02422675
    ## SRR1039512 GSM1275866 N052611 untrt untrt SRR1039512       126  SRX384349 SRS508571 SAMN02422678
    ## SRR1039513 GSM1275867 N052611   trt untrt SRR1039513        87  SRX384350 SRS508572 SAMN02422670
    ## SRR1039516 GSM1275870 N080611 untrt untrt SRR1039516       120  SRX384353 SRS508575 SAMN02422682
    ## SRR1039517 GSM1275871 N080611   trt untrt SRR1039517       126  SRX384354 SRS508576 SAMN02422673
    ## SRR1039520 GSM1275874 N061011 untrt untrt SRR1039520       101  SRX384357 SRS508579 SAMN02422683
    ## SRR1039521 GSM1275875 N061011   trt untrt SRR1039521        98  SRX384358 SRS508580 SAMN02422677

## Counting reads in genes

Once the reads have been aligned, there are a number of tools which
can be used to count the number of reads which can be unambiguously
assigned to genomic features for each sample. These often take as
input SAM/BAM alignment files and a file specifiying the genomic
features, e.g. a GFF3 or GTF file specifying the gene models.

The following tools can be used generate count matrices:

function            | package                                              | framework      | output                 | *DESeq2* input function
--------------------|------------------------------------------------------|----------------|------------------------|-------------------------
*summarizeOverlaps* | *[GenomicAlignments](http://bioconductor.org/packages/release/bioc/html/GenomicAlignments.html)*                     | R/Bioconductor | *SummarizedExperiment* | *DESeqDataSet*
*featureCounts*     | *[Rsubread](http://bioconductor.org/packages/release/bioc/html/Rsubread.html)*                              | R/Bioconductor | matrix                 | *DESeqDataSetFromMatrix*
*htseq-count*       | [HTSeq](http://www-huber.embl.de/users/anders/HTSeq) | Python         | files                  | *DESeqDataSetFromHTSeq* 

Using the `Run` column in the sample table, we construct the full
paths to the files we want to perform the counting operation on:


    filenames <- file.path(dir, paste0(sampleTable$Run, "_subset.bam"))

We indicate in Bioconductor that these files are BAM files using the
*BamFileList* function. Here we also specify details about how the BAM files should
be treated, e.g., only process 2000000 reads at a time.


    library("Rsamtools")
    bamfiles <- BamFileList(filenames, yieldSize=2000000)

**Note:** make sure that the chromosome names of the genomic features
in the annotation you use are consistent with the chromosome names of
the reference used for read alignment. Otherwise, the scripts might
fail to count any reads to features due to the mismatching names.
We can check the chromosome names in the alignment files like so:


    seqinfo(bamfiles[1])

    ## Seqinfo object with 84 sequences from an unspecified genome:
    ##   seqnames   seqlengths isCircular genome
    ##   1           249250621       <NA>   <NA>
    ##   10          135534747       <NA>   <NA>
    ##   11          135006516       <NA>   <NA>
    ##   12          133851895       <NA>   <NA>
    ##   13          115169878       <NA>   <NA>
    ##   ...               ...        ...    ...
    ##   GL000210.1      27682       <NA>   <NA>
    ##   GL000231.1      27386       <NA>   <NA>
    ##   GL000229.1      19913       <NA>   <NA>
    ##   GL000226.1      15008       <NA>   <NA>
    ##   GL000207.1       4262       <NA>   <NA>

Next, we need to read in the gene model which will be used for
counting reads. We will read the gene model from a
[GTF file](http://www.ensembl.org/info/website/upload/gff.html), using
*makeTranscriptDbFromGFF* from the *[GenomicFeatures](http://bioconductor.org/packages/release/bioc/html/GenomicFeatures.html)*
package. GTF files can be downloaded from
Ensembl's FTP site or other gene model repositories.
A *TranscriptDb* object is a database which can be used to
generate a variety of range-based objects, such as exons, transcripts,
and genes. We will want to make a list of exons grouped by gene.

There are other options for constructing a *TranscriptDB*.
For the *known genes* track from the UCSC Genome Browser,
one can use the pre-built Transcript DataBase:
*[TxDb.Hsapiens.UCSC.hg19.knownGene](http://bioconductor.org/packages/release/data/annotation/html/TxDb.Hsapiens.UCSC.hg19.knownGene.html)*.
The *makeTranscriptDbFromBiomart* function can be used to automatically
pull a gene model from Biomart. 


    library("GenomicFeatures")


    gtffile <- file.path(dir,"Homo_sapiens.GRCh37.75_subset.gtf")
    (txdb <- makeTranscriptDbFromGFF(gtffile, format="gtf"))

    ## Warning in .deduceExonRankings(exs, format = "gtf"): Infering Exon Rankings.  If this is not what
    ## you expected, then please be sure that you have provided a valid attribute for
    ## exonRankAttributeName

    ## Warning in matchCircularity(chroms, circ_seqs): None of the strings in your circ_seqs argument
    ## match your seqnames.

    ## TxDb object:
    ## | Db type: TxDb
    ## | Supporting package: GenomicFeatures
    ## | Data source: /var/lib/jenkins/R/x86_64-unknown-linux-gnu-library/3.1/airway/extdata/Homo_sapiens.GRCh37.75_subset.gtf
    ## | Organism: NA
    ## | miRBase build ID: NA
    ## | transcript_nrow: 65
    ## | exon_nrow: 242
    ## | cds_nrow: 159
    ## | Db created by: GenomicFeatures package from Bioconductor
    ## | Creation time: 2014-10-23 13:51:14 -0700 (Thu, 23 Oct 2014)
    ## | GenomicFeatures version at creation time: 1.18.1
    ## | RSQLite version at creation time: 0.11.4
    ## | DBSCHEMAVERSION: 1.0

The following line produces a *GRangesList* of all the exons grouped by gene.


    (genes <- exonsBy(txdb, by="gene"))

    ## GRangesList object of length 20:
    ## $ENSG00000009724 
    ## GRanges object with 16 ranges and 2 metadata columns:
    ##        seqnames               ranges strand   |   exon_id   exon_name
    ##           <Rle>            <IRanges>  <Rle>   | <integer> <character>
    ##    [1]        1 [11086580, 11087705]      -   |        89        <NA>
    ##    [2]        1 [11090233, 11090307]      -   |        90        <NA>
    ##    [3]        1 [11090805, 11090939]      -   |        91        <NA>
    ##    [4]        1 [11094885, 11094963]      -   |        92        <NA>
    ##    [5]        1 [11097750, 11097868]      -   |        93        <NA>
    ##    ...      ...                  ...    ... ...       ...         ...
    ##   [12]        1 [11106613, 11106790]      -   |       100        <NA>
    ##   [13]        1 [11106948, 11107176]      -   |       101        <NA>
    ##   [14]        1 [11107260, 11107280]      -   |       102        <NA>
    ##   [15]        1 [11107260, 11107284]      -   |       103        <NA>
    ##   [16]        1 [11107260, 11107290]      -   |       104        <NA>
    ## 
    ## ...
    ## <19 more elements>
    ## -------
    ## seqinfo: 1 sequence from an unspecified genome; no seqlengths

After these preparations, the actual counting is easy. The function
*summarizeOverlaps* from the *[GenomicAlignments](http://bioconductor.org/packages/release/bioc/html/GenomicAlignments.html)*
package will do this. This produces a *SummarizedExperiment*
object, which contains a variety of information about
an experiment, and will be described in more detail below.

**Note:** If it is desired to perform counting using multiple cores, one can use
the *register* and *MulticoreParam* functions from the
*[BiocParallel](http://bioconductor.org/packages/release/bioc/html/BiocParallel.html)* package before the counting call below.


    library("GenomicAlignments")


    se <- summarizeOverlaps(features=genes, reads=bamfiles,
                            mode="Union",
                            singleEnd=FALSE,
                            ignore.strand=TRUE,
                            fragments=TRUE )

We specify a number of arguments besides the `features` and the
`reads`. The `mode` argument describes what kind of read overlaps will
be counted as a hit. These modes are shown in Figure 1 of the "Counting reads with
summarizeOverlaps" vignette for the *[GenomicAlignments](http://bioconductor.org/packages/release/bioc/html/GenomicAlignments.html)*
package. Setting `singleEnd` to `FALSE` indicates that the experiment
produced paired-end reads, and we want to count a pair of reads only once
toward the read count for a gene.
In order to produce correct counts, it is important to know if the
RNA-Seq experiment was strand-specific or not. This experiment was not
strand-specific so we set `ignore.strand` to `FALSE`.
The `fragments` argument can be used when `singleEnd=FALSE`
to specify if unpaired reads should be counted (yes if `fragments=TRUE`).





































































































































